function [xMR,minRes,iterMR] = nlcgClever(f,grad,n,NAtb,maxIter)
% *** Non-linear Conjugate Gradient method ***
% Fletcher-Reeves with modified backtricking line search method
% INPUT:
% f - function to minimalize
% grad  - gradient of the function f
% n - size of the vector x
% NAtb = norm(A'*b)
% maxIter - maximum number of iterations
% OUTPUT:
% xMR - solution of NLCG method   (or solution with min relative residuum 
%       when maxIter is reached)
% minRes - relative residual of solution
% iterMR - the iteration when min rel residuum has been reached
% % written by Jakub Sevcik and Vaclav Smidl 2016
tic;

% default value
x1 = zeros(n,1);
tol = 1e-4;
maxIter;

g1 = grad(x1);
p1 = -g1;
res = Inf;
iter = 1;

minRes = Inf;
iterMR = 0;
xMR = [];

while res > tol
    g0 = g1;
    x0 = x1;
    p0 = p1;
    
    % Backtracking line search
    t = -0.5*p0'*g0;
    const1 = 1;
    fx0 = f(x0);
    
    k = 1;
    % extra condition k<120 : const1 > (0.75)^120
    % default backtracking LS: while fx0-f(x0+const1*p0) < const1*t
    while fx0-f(x0+const1*p0) < const1*t && k<120; 
     k = k+1;
     const1 = 0.75*const1;
    end
    % end line search
    
    x1 = x0 + const1*p0;
    
    res = norm(grad(x1))/NAtb;
    if res < minRes
        minRes = res;
        iterMR = iter;
        xMR = x1;
    end
    %%% reflecting of negative value
    %index = x1<0;
    %x1(index) = -x1(index);
    
    g1 = grad(x1);
    const2 = (g1'*g1)/(g0'*g0);
    p1 = -g1 + const2*p0;
    iter = iter+1;
    
    if iter > maxIter
        time = toc;
        fprintf(['nlcg stopped at iteration ', int2str(iter-1),' without converging to the desired tolerance ',...
        num2str(tol,2), '\n', 'because the maximum number of iterations was reached .\n',...
        'The iterate returned (number ', int2str(iterMR), ') has relative residual ', num2str(minRes,2),...
        ' at time ', num2str(time), '\n']);
        % xMR=x1; % for returning the solution from last iteration (not
        %         % with min residuum)
        return;
    end
end
time = toc;
disp(['nlcg converged at iteration ', int2str(iter-1),' to a solution with relative residual ',...
    num2str(res,2), ' at time ', num2str(time)]);
end

